//
//  ViewController.m
//  ifreeSampleObjC_13
//
//  Created by HeeJun Shin on 2022/07/05.
//


#import "ViewController.h"

#import "KeychainItemWrapper.h"

@interface ViewController ()

@property (strong, nonatomic) IBOutlet UITextView *tvInfoView;

@end

@implementation ViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view.
}

#pragma mark - IBAction
- (IBAction) actionConnectWithifree:(id)sender
{
    //파트너사가 생성한 token
    self.tvInfoView.text = @"";
    NSString *partnerStateToken = [self createStatToken];
    [[ifreeManager getInstance] login:self withStateToken:partnerStateToken];
    
    

}

- (IBAction) actionConnectWithifreePresent:(id)sender
{
    //파트너사가 생성한 token

    self.tvInfoView.text = @"";
    UIStoryboard *mainStoryboard = [UIStoryboard storyboardWithName:@"Main" bundle:[NSBundle mainBundle]];

    LoginViewController *vc = (LoginViewController*)[mainStoryboard instantiateViewControllerWithIdentifier:@"LoginViewController"];
    vc.delegate = self;
    [self presentViewController:vc animated:true completion:^{
        NSLog(@"ifreeWebViewController close");
    }];

}


- (void) actionPartnerLoginWithifreeAuth
{
    // 화면 처리
    //TODO:

    NSLog(@"actionPartnerLoginWithifreeAuth");
    if( [[ifreeManager getInstance] getReciveStateToken] == nil)
    {
        [self showMessage:@"getReciveStateToken empty"];
        return;
    }
    
    if(![[[ifreeManager getInstance] getStateToken] isEqualToString: [[ifreeManager getInstance] getReciveStateToken]])
    {
        [self showMessage:@"getStateToken not same"];
        return;
    }
    
    if( [[ifreeManager getInstance] getPartnerToken] == nil)
    {
        [self showMessage:@"getPartnerToken empty"];
        return;
    }
    
    NSMutableDictionary *loginDic = [NSMutableDictionary dictionary];
    [loginDic setValue: [[ifreeManager getInstance] getClientId] forKey:@"client_id"];
    [loginDic setValue: [[ifreeManager getInstance] getPartnerToken] forKey:@"ptn_token"];
    // 각 제휴사 마다 다름
    [[ifreeManager getInstance] partnerLoginCheck:loginDic withCompletion:^(NSDictionary* result, NSError * _Nonnull error) {
        [[ifreeManager getInstance] closeAuthResult];
        
        if( [[result objectForKey:@"code"] isEqualToString:@"0000"] )
        {
            [self showMessage:@"파트너사 로그인 성공 "];
        }
        else
        {
            [self showMessage:[NSString stringWithFormat: @"파트너사 로그인 에러 처리 ( %@ )",[result objectForKey:@"code"]]];
        }
    }];
}



#pragma mark -
- (void) showMessage:(NSString *)message
{
    UIAlertController * alertController=   [UIAlertController
                                            alertControllerWithTitle:nil
                                            message:message
                                            preferredStyle:UIAlertControllerStyleAlert];
    
    UIAlertAction *okAction = [UIAlertAction actionWithTitle:@"확인" style:UIAlertActionStyleDefault handler:nil];
    
    [alertController addAction:okAction];
    
    [self presentViewController:alertController animated:YES completion:nil];
}

#pragma mark - LoginViewControllerDelegate
- (void)loginInfo:(NSString*)strInfo
{
    NSLog(@"loginInfo = %@",strInfo);
    self.tvInfoView.text = strInfo;
}

#pragma mark - ifreeWebViewControllerDelegate

- (void)onCallback:(NSMutableDictionary *)successAuthValue withErrInfo: (NSMutableDictionary *)errInfo
{
    NSLog(@"ViewCongtroller onCallback %@ ",successAuthValue);
    NSLog(@"ViewCongtroller onCallback  errInfo%@ ",errInfo);
    // result_code: sdk내부 정의 값
    // err_code: 패시키 앱에서 내려주는 코드(에러)
    
    if(successAuthValue != nil)
    {
        self.tvInfoView.text = [NSString stringWithFormat:@"ResultValue:%@\nptn_token:%@\nstat_token:%@",[successAuthValue objectForKey:@"result_code"],[successAuthValue objectForKey:@"ptn_token"],[successAuthValue objectForKey:@"stat_token"]];
        [self actionPartnerLoginWithifreeAuth];
    }
    else
    {
        
        if ([errInfo objectForKey:@"result_code"] != nil)
        {
            NSString *message;
            if([errInfo objectForKey:@"err_code"] != nil)
            {
                message = [NSString stringWithFormat:@"ResultValue: %@(%@)",[errInfo objectForKey:@"result_code"],[errInfo objectForKey:@"err_code"]];
            }
            else
            {
                message = [NSString stringWithFormat:@"ResultValue: %@",[errInfo objectForKey:@"result_code"]];
            }
            self.tvInfoView.text = message;
           
        }
        else if ([errInfo objectForKey:@"result_code"] == nil &&  [errInfo objectForKey:@"err_code"] != nil)
        {
            NSString *message = [NSString stringWithFormat:@"ResultValue: %@",[errInfo objectForKey:@"err_code"]];
            self.tvInfoView.text = message;
        }
        else if ([errInfo objectForKey:@"code"] != nil)
        {
            NSString *message = [NSString stringWithFormat:@"ResultValue: %@",[errInfo objectForKey:@"code"]];
            self.tvInfoView.text = message;
        }
    }
    
    
}


- (NSString*) createStatToken

{
    // initialize keychaing item for saving UUID.
    KeychainItemWrapper *wrapper = [[KeychainItemWrapper alloc] initWithIdentifier:@"UUID" accessGroup:nil];
    NSString *uuid = [wrapper objectForKey:(__bridge id)(kSecAttrAccount)];
    if( uuid == nil || uuid.length == 0)
    {

        // if there is not UUID in keychain, make UUID and save it.

        CFUUIDRef uuidRef = CFUUIDCreate(NULL);

        CFStringRef uuidStringRef = CFUUIDCreateString(NULL, uuidRef);

        CFRelease(uuidRef);

        uuid = [NSString stringWithString:(__bridge NSString *) uuidStringRef];

        CFRelease(uuidStringRef);

        // save UUID in keychain

        //[wrapper setObject:uuid forKey:(__bridge id)(kSecAttrAccount)];

    }

    NSLog(@"cragtePartnerStatToken = %@",uuid);

    return uuid;

}


@end
